@@ -1,17 +1,17 @@
 # PGP::Sign -- Create a PGP signature for data, securely.  -*- perl -*-
-# $Id: Sign.in,v 0.14 2000/02/12 12:21:30 eagle Exp $
+# $Id: Sign.in,v 0.15 2002/06/28 23:27:20 eagle Exp $
 #
-# Copyright 1997, 1998, 1999, 2000 by Russ Allbery <rra@stanford.edu>
+# Copyright 1997, 1998, 1999, 2000, 2002 Russ Allbery <rra@stanford.edu>
 #
-# This program is free software; you can redistribute it and/or modify it
+# This program is free software; you may redistribute it and/or modify it
 # under the same terms as Perl itself.
 #
 #                     THIS IS NOT A GENERAL PGP MODULE
 #
 # For a general PGP module that handles encryption and decryption, key ring
 # management, and all of the other wonderful things you want to do with PGP,
-# see the PGP module directory on CPAN.  This module is designed to do one
-# and only one thing and do it fast, well, and securely -- create and check
+# see the PGP module directory on CPAN.  This module is designed to do one and
+# only one thing and do it fast, well, and securely -- create and check
 # detached signatures for some block of data.
 #
 # This above all: to thine own self be true,
@@ -19,9 +19,9 @@
 # Thou canst not then be false to any man.
 #                               -- William Shakespeare, _Hamlet_
 
-############################################################################
+##############################################################################
 # Modules and declarations
-############################################################################
+##############################################################################
 
 package PGP::Sign;
 require 5.003;
@@ -41,12 +41,11 @@ use vars qw(@ERROR @EXPORT @EXPORT_OK @ISA $MUNGE $PGPS $PGPV $PGPPATH
 
 # The current PGP::Sign version number, which is not the same as the CVS
 # revision of this file.  Picked up during build time.
-$VERSION = '0.16';
+$VERSION = '0.17';
 
-
-############################################################################
+##############################################################################
 # Global variables
-############################################################################
+##############################################################################
 
 # The text of any errors resulting from the last call to pgp_sign().
 @ERROR = ();
@@ -61,23 +60,22 @@ $MUNGE = 0;
 $PGPS = '/usr/local/bin/gpg';
 $PGPV = '/usr/local/bin/gpg';
 
-# The path to the directory containing the key ring.  If not set, PGP
-# (v2, v5, or v6) defaults to $ENV{PGPPATH} or $HOME/.pgp.  GPG defaults
-# to $ENV{GNUPGHOME} or $HOME/.gnupg.  Setting this will also be made to
-# affect GPG.
+# The path to the directory containing the key ring.  If not set, PGP (v2, v5,
+# or v6) defaults to $ENV{PGPPATH} or $HOME/.pgp.  GPG defaults to
+# $ENV{GNUPGHOME} or $HOME/.gnupg.  Setting this will also be made to affect
+# GPG.
 $PGPPATH = '';
 
-# What style of PGP invocation to use by default.  Allowable values are
-# PGP2, PGP5, PGP6, and GPG.  This is picked up at install time.
+# What style of PGP invocation to use by default.  Allowable values are PGP2,
+# PGP5, PGP6, and GPG.  This is picked up at install time.
 $PGPSTYLE = 'GPG';
 
 # The directory in which temporary files should be created.
 $TMPDIR = $ENV{TMPDIR} || '/tmp';
 
-
-############################################################################
+##############################################################################
 # Implementation
-############################################################################
+##############################################################################
 
 # This function actually sends the data to a file handle.  It's necessary to
 # implement munging (stripping trailing spaces on a line).
@@ -99,16 +97,16 @@ $TMPDIR = $ENV{TMPDIR} || '/tmp';
 }
 
 # This is our generic "take this data and shove it" routine, used both for
-# signature generation and signature checking.  The first argument is the
-# file handle to shove all the data into, and the remaining arguments are
-# sources of data.  Scalars, references to arrays, references to FileHandle
-# or IO::Handle objects, file globs, references to code, and references to
-# file globs are all supported as ways to get the data, and at most one line
-# at a time is read (cutting down on memory usage).
+# signature generation and signature checking.  The first argument is the file
+# handle to shove all the data into, and the remaining arguments are sources
+# of data.  Scalars, references to arrays, references to FileHandle or
+# IO::Handle objects, file globs, references to code, and references to file
+# globs are all supported as ways to get the data, and at most one line at a
+# time is read (cutting down on memory usage).
 #
-# References to code are an interesting subcase.  A code reference is
-# executed repeatedly, whatever it returns being passed to PGP using the ORS
-# specified if any, until it returns undef.
+# References to code are an interesting subcase.  A code reference is executed
+# repeatedly, whatever it returns being passed to PGP using the ORS specified
+# if any, until it returns undef.
 sub write_data {
     my $fh = shift;
 
@@ -117,11 +115,10 @@ sub write_data {
     # Unfortunately, we can't rely on 5.004 yet.  *But*, the main reason we
     # want isa() is to handle the various derived IO::Handle classes, and
     # 5.003 should only have FileHandle, so we can hack our way around that.
-    # We can't do anything interesting or particularly "cool" with
-    # references to references, so those we just print.  (Perl allows
-    # circular references, so we can't just dereference references to
-    # references until we get something interesting.)  Hashes are treated
-    # like arrays.
+    # We can't do anything interesting or particularly "cool" with references
+    # to references, so those we just print.  (Perl allows circular
+    # references, so we can't just dereference references to references until
+    # we get something interesting.)  Hashes are treated like arrays.
     my $source;
     for $source (@_) {
         if (ref $source eq 'ARRAY' or ref $source eq 'HASH') {
@@ -188,14 +185,14 @@ sub pgp_sign {
         @command = ($PGPS, '+batchmode', '-sbaft', '-u', $keyid);
     }
 
-    # We need to send the password to PGP, but we don't want to use either
-    # the command line or an environment variable, since both may expose us
-    # to snoopers on the system.  So we create a pipe, stick the password in
-    # it, and then pass the file descriptor to PGP.  PGP wants to know about
-    # this in an environment variable; GPG uses a command-line flag.
-    # 5.005_03 started setting close-on-exec on file handles > $^F, so we
-    # need to clear that here (but ignore errors on platforms where fcntl or
-    # F_SETFD doesn't exist, if any).
+    # We need to send the password to PGP, but we don't want to use either the
+    # command line or an environment variable, since both may expose us to
+    # snoopers on the system.  So we create a pipe, stick the password in it,
+    # and then pass the file descriptor to PGP.  PGP wants to know about this
+    # in an environment variable; GPG uses a command-line flag.  5.005_03
+    # started setting close-on-exec on file handles > $^F, so we need to clear
+    # that here (but ignore errors on platforms where fcntl or F_SETFD doesn't
+    # exist, if any).
     my $passfh = new FileHandle;
     my $writefh = new FileHandle;
     pipe ($passfh, $writefh);
@@ -209,9 +206,9 @@ sub pgp_sign {
         $ENV{PGPPASSFD} = $passfh->fileno ();
     }
 
-    # Fork off a pgp process that we're going to be feeding data to, and
-    # tell it to just generate a signature using the given key id and pass
-    # phrase.  Set PGPPATH if desired.
+    # Fork off a pgp process that we're going to be feeding data to, and tell
+    # it to just generate a signature using the given key id and pass phrase.
+    # Set PGPPATH if desired.
     local $ENV{PGPPATH} = $PGPPATH if ($PGPPATH && $PGPSTYLE ne 'GPG');
     if ($PGPPATH && $PGPSTYLE eq 'GPG') {
         push (@command, '--homedir', $PGPPATH);
@@ -229,8 +226,8 @@ sub pgp_sign {
     unshift (@_, $pgp);
     &write_data;
 
-    # All done.  Close the pipe to PGP, clean up, and see if we succeeded.
-    # If not, save the error output and return undef.
+    # All done.  Close the pipe to PGP, clean up, and see if we succeeded.  If
+    # not, save the error output and return undef.
     close $pgp;
     local $/ = "\n";
     my @errors = <$errors>;
@@ -245,9 +242,9 @@ sub pgp_sign {
     }
 
     # Now, clean up the returned signature and return it, along with the
-    # version number if desired.  PGP v2 calls this a PGP MESSAGE, whereas
-    # PGP v5 and v6 and GPG both (more correctly) call it a PGP SIGNATURE,
-    # so accept either.
+    # version number if desired.  PGP v2 calls this a PGP MESSAGE, whereas PGP
+    # v5 and v6 and GPG both (more correctly) call it a PGP SIGNATURE, so
+    # accept either.
     while ((shift @signature) !~ /-----BEGIN PGP \S+-----\n/) {
         unless (@signature) {
             @ERROR = ("No signature from PGP (command not found?)\n");
@@ -266,12 +263,12 @@ sub pgp_sign {
     wantarray ? ($signature, $version) : $signature;
 }
 
-# Check a detatched signature for given data.  Takes a signature block (in
-# the form of an ASCII-armored string with embedded newlines), a version
-# number (which may be undef), and some number of data sources that
-# write_data() can handle and returns the key id of the signature, the empty
-# string if the signature didn't check, and undef in the event of an error.
-# In the event of some sort of an error, we stick the error in @ERROR.
+# Check a detatched signature for given data.  Takes a signature block (in the
+# form of an ASCII-armored string with embedded newlines), a version number
+# (which may be undef), and some number of data sources that write_data() can
+# handle and returns the key id of the signature, the empty string if the
+# signature didn't check, and undef in the event of an error.  In the event of
+# some sort of an error, we stick the error in @ERROR.
 sub pgp_verify {
     my $signature = shift;
     my $version = shift;
@@ -280,13 +277,13 @@ sub pgp_verify {
     # Ignore SIGPIPE, since we're going to be talking to PGP.
     local $SIG{PIPE} = 'IGNORE';
 
-    # Because this is a detached signature, we actually need to save both
-    # the signature and the data to files and then run PGP on the signature
-    # file to make it verify the signature.  Because this is a detached
-    # signature, though, we don't have to do any data mangling, which makes
-    # our lives much easier.  It would be nice to do this without having to
-    # use temporary files, but I don't see any way to do so without running
-    # into mangling problems.
+    # Because this is a detached signature, we actually need to save both the
+    # signature and the data to files and then run PGP on the signature file
+    # to make it verify the signature.  Because this is a detached signature,
+    # though, we don't have to do any data mangling, which makes our lives
+    # much easier.  It would be nice to do this without having to use
+    # temporary files, but I don't see any way to do so without running into
+    # mangling problems.
     #
     # PGP v5 *requires* there be some subheader or another.  *sigh*  So we
     # supply one if Version isn't given.  :)
@@ -332,10 +329,10 @@ sub pgp_verify {
         @command = ($PGPV, '+batchmode');
     }
 
-    # Now, call PGP to check the signature.  Because we've written
-    # everything out to a file, this is actually fairly simple; all we need
-    # to do is grab stdout.  PGP prints its banner information to stderr, so
-    # just ignore stderr.  Set PGPPATH if desired.
+    # Now, call PGP to check the signature.  Because we've written everything
+    # out to a file, this is actually fairly simple; all we need to do is grab
+    # stdout.  PGP prints its banner information to stderr, so just ignore
+    # stderr.  Set PGPPATH if desired.
     local $ENV{PGPPATH} = $PGPPATH if ($PGPPATH && $PGPSTYLE ne 'GPG');
     if ($PGPPATH && $PGPSTYLE eq 'GPG') {
         push (@command, '--homedir', $PGPPATH);
@@ -355,8 +352,8 @@ sub pgp_verify {
     # Check for the message that gives us the key status and return the
     # appropriate thing to our caller.  This part is a zoo due to all of the
     # different formats used.  GPG has finally done the right thing and
-    # implemented a separate status stream with parseable data, but I
-    # haven't gotten to implementing that yet....
+    # implemented a separate status stream with parseable data, but I haven't
+    # gotten to implementing that yet....
     #
     # Partly from pgpverify, by David Lawrence <tale@isc.org>:
     #
@@ -399,16 +396,15 @@ sub pgp_verify {
     $signer ? $signer : '';
 }
 
-# Return the errors resulting from the last call to pgp_sign() or
-# pgp_verify() or the empty list if there are none.
+# Return the errors resulting from the last call to pgp_sign() or pgp_verify()
+# or the empty list if there are none.
 sub pgp_error {
     wantarray ? @ERROR : join ('', @ERROR);
 }
 
-
-############################################################################
+##############################################################################
 # Module return value and documentation
-############################################################################
+##############################################################################
 
 # Make sure the module returns true.
 1;
@@ -480,9 +476,9 @@ by the last pgp_sign() or pgp_verify(), or undef if there was no error.  In
 a list context, a list of lines is returned; in a scalar context, a long
 string with embedded newlines is returned.
 
-Six global variables can be modified (note that these will likely eventually
-be partially or completely replaced with an interface via a
-PGP::Sign::config() call):
+Six global variables can be modified (note that these may eventually be
+partially or completely replaced with an interface via a PGP::Sign::config()
+call):
 
 =over 4
 
@@ -624,12 +620,17 @@ PGP behavior instead.
 
 =head1 CAVEATS
 
-This module is fairly good at what it does, but it doesn't as yet do very
-much.  I intend to provide more options and more configurability in the
-future, particularly the ability to handle binary data, but that may mean
-API changes.  The interface to this module should not be considered stable
-yet; you may have to change your application when you upgrade to a newer
-version of this module.  The README will list API changes.
+This module is fairly good at what it does, but it doesn't do very much.  At
+one point, I had plans to provide more options and more configurability in
+the future, particularly the ability to handle binary data, that would
+probably mean API changes.  I'm not sure at this point whether I'll get to
+that, or just replace this module with one that only uses GnuPG as I see no
+reason to use any other PGP implementation at this point and GnuPG has a
+much nicer programmatic interface.
+
+However, just in case, the interface to this module should not be considered
+stable yet; you may have to change your application when you upgrade to a
+newer version of this module.  The README will list API changes.
 
 =head1 RESTRICTIONS
 
@@ -645,27 +646,39 @@ substandard operating systems.
 
 =head1 SEE ALSO
 
-pgp(1), pgps(1), pgpv(1), gpg(1), RFC 2440
+pgp(1), pgps(1), pgpv(1), gpg(1)
+
+RFC 2440, L<http://www.rfc-editor.org/rfc/rfc2440.txt>, which specifies the
+OpenPGP message format.
+
+The current version of this module is always available from its web site at
+L<http://www.eyrie.org/~eagle/software/pgp-sign/>.
 
 =head1 AUTHOR
 
-Russ Allbery E<lt>rra@stanford.eduE<gt>.
+Russ Allbery <rra@stanford.edu>
+
+=head1 COPYRIGHT AND LICENSE
+
+Copyright 1997, 1998, 1999, 2000, 2002 Russ Allbery <rra@stanford.edu>
+
+This program is free software; you may redistribute it and/or modify it
+under the same terms as Perl itself.
 
 =head1 HISTORY
 
-Based heavily on work by Andrew Gierth
-E<lt>andrew@erlenstar.demon.co.ukE<gt> and benefitting greatly from input,
-comments, suggestions, and help from him, this module came about in the
-process of implementing PGPMoose signatures and control message signatures
-for Usenet.  PGPMoose is the idea of Greg Rose E<lt>ggr@usenix.orgE<gt>, and
-signcontrol and pgpverify are the idea of David Lawrence
-E<lt>tale@isc.orgE<gt>.
+Based heavily on work by Andrew Gierth <andrew@erlenstar.demon.co.uk> and
+benefitting greatly from input, comments, suggestions, and help from him,
+this module came about in the process of implementing PGPMoose signatures
+and control message signatures for Usenet.  PGPMoose is the idea of Greg
+Rose <ggr@usenix.org>, and signcontrol and pgpverify are the idea of David
+Lawrence <tale@isc.org>.
 
 Support for PGPPATH, the test suite, some bug fixes, and the impetus to get
-another version released came from Andrew Ford
-E<lt>A.Ford@ford-mason.co.ukE<gt>.  Thank you.
+another version released came from Andrew Ford <A.Ford@ford-mason.co.uk>.
+Thank you.
 
-Original support for GnuPG from Todd Underwood E<lt>todd@nm.netE<gt> and
-Monte Mitzelfelt E<lt>monte@nm.netE<gt>.
+Original support for GnuPG from Todd Underwood <todd@nm.net> and Monte
+Mitzelfelt <monte@nm.net>.
 
 =cut
